@@ -29,6 +29,7 @@
 
 #ifdef HAVE_AVAHI
 #include <avahi-client/client.h>
+#include <avahi-client/publish.h>
 #include <avahi-common/alternative.h>
 #include <avahi-common/error.h>
 #include <avahi-glib/glib-watch.h>
@@ -66,6 +67,86 @@ static GMainLoop *loop = NULL;
 static pid_t httpd_pid = 0;
 static guint disabled_timeout_tag = 0;
 
+static char *
+lookup_public_dir (void)
+{
+  FILE *file;
+  char *config_file;
+  char buffer[512];
+  char *user_dir;
+  char *p, *d;
+  int len;
+  int relative;
+  
+  config_file = g_build_filename (g_get_user_config_dir (), "user-dirs.dirs", NULL);
+  file = fopen (config_file, "r");
+  free (config_file);
+  if (file == NULL)
+	  goto error;
+  
+  user_dir = NULL;
+  while (fgets (buffer, sizeof (buffer), file)) {
+	  /* Remove newline at end */
+	  len = strlen (buffer);
+      if (len > 0 && buffer[len-1] == '\n')
+		  buffer[len-1] = 0;
+      
+      p = buffer;
+      while (*p == ' ' || *p == '\t')
+		  p++;
+      
+      if (!g_str_has_prefix (p, "XDG_PUBLICSHARE_DIR"))
+		  continue;
+      p += strlen ("XDG_PUBLICSHARE_DIR");
+	  
+      while (*p == ' ' || *p == '\t')
+		  p++;
+	  
+      if (*p != '=')
+		  continue;
+      p++;
+      
+      while (*p == ' ' || *p == '\t')
+		  p++;
+	  
+      if (*p != '"')
+		  continue;
+      p++;
+      
+      relative = 0;
+      if (strncmp (p, "$HOME/", 6) == 0) {
+		  p += 6;
+		  relative = 1;
+	  }
+      else if (*p != '/')
+		  continue;
+      
+      if (relative)	{
+		  user_dir = g_malloc (strlen (g_get_home_dir()) + 1 + strlen (p) + 1);
+		  strcpy (user_dir, g_get_home_dir ());
+		  strcat (user_dir, "/");
+	  } else {
+		  user_dir = g_malloc (strlen (p) + 1);
+		  *user_dir = 0;
+	  }
+      
+      d = user_dir + strlen (user_dir);
+      while (*p && *p != '"') {
+		  if ((*p == '\\') && (*(p+1) != 0))
+			  p++;
+		  *d++ = *p++;
+	  }
+      *d = 0;
+  }  
+  fclose (file);
+  
+  if (user_dir)
+	  return user_dir;
+  
+ error:
+  return g_build_filename (g_get_home_dir (), "Public", NULL);
+}
+
 static int
 get_port (void)
 {
@@ -111,16 +192,32 @@ get_port (void)
 static char *
 get_share_name (void)
 {
-	/* Translators: The %s will get filled in with the user name
-	   of the user, to form a genitive. If this is difficult to
-	   translate correctly so that it will work correctly in your
-	   language, you may use something equivalent to
-	   "Public files of %s", or leave out the %s altogether.
-	   In the latter case, please put "%.0s" somewhere in the string,
-	   which will match the user name string passed by the C code,
-	   but not put the user name in the final string. This is to
-	   avoid the warning that msgfmt might otherwise generate. */
-    return g_strdup_printf (_("%s's public files"), g_get_user_name ());
+	static char *name = NULL;
+	const char *host_name;
+	
+	if (name == NULL) {
+		host_name = g_get_host_name ();
+		if (strcmp (host_name, "localhost") == 0) {
+			/* Translators: The %s will get filled in with the user name
+			   of the user, to form a genitive. If this is difficult to
+			   translate correctly so that it will work correctly in your
+			   language, you may use something equivalent to
+			   "Public files of %s", or leave out the %s altogether.
+			   In the latter case, please put "%.0s" somewhere in the string,
+			   which will match the user name string passed by the C code,
+			   but not put the user name in the final string. This is to
+			   avoid the warning that msgfmt might otherwise generate. */
+			name = g_strdup_printf (_("%s's public files"), g_get_user_name ());
+		} else {
+			/* Translators: This is similar to the string before, only it
+			   has the hostname in it too. */
+			name = g_strdup_printf (_("%s's public files on %s"),
+									g_get_user_name (),
+									host_name);
+		}
+			
+	}
+	return name;
 }
 
 
@@ -143,6 +240,7 @@ create_service (void) {
 	}
 
 	ret = avahi_entry_group_add_service (entry_group, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, 
+										 AVAHI_PUBLISH_USE_MULTICAST,
 										 avahi_name, "_webdav._tcp", NULL, NULL, 
 										 avahi_port, "u=guest", NULL);
 	if (ret < 0) {
@@ -189,7 +287,7 @@ avahi_client_callback (AvahiClient *client, AvahiClientState state, void *userda
 		}
 	} else if (state == AVAHI_CLIENT_S_COLLISION) {
 		avahi_entry_group_reset (entry_group);
-    } else if (state == AVAHI_CLIENT_DISCONNECTED) {
+    } else if (state == AVAHI_CLIENT_FAILURE) {
 		avahi_running = FALSE;
     }
 }
@@ -206,6 +304,7 @@ init_avahi (void)
 
 	/* Create a new AvahiClient instance */
 	avahi_client = avahi_client_new (avahi_glib_poll_get (poll),
+									 AVAHI_CLIENT_NO_FAIL,
 									 avahi_client_callback,
 									 NULL,
 									 &error);
@@ -301,7 +400,6 @@ publish_service (int port)
 								   /* TODO: should be u=guest */
 								   /* text */ (unsigned char *) "", 0,
 								   publish_reply, NULL, &published_id);
-    g_free (share_name);
     if (result != SW_OKAY) {
 		return FALSE;
     }
@@ -319,15 +417,11 @@ stop_publishing (void)
 
 
 static void
-ensure_public_dir (void)
+ensure_public_dir (char *dirname)
 {
-    char *dirname;
-
-    dirname = g_build_filename (g_get_home_dir (), "Public", NULL);
     if (!g_file_test (dirname, G_FILE_TEST_IS_DIR)) {
 		mkdir (dirname, 0755);
     }
-    g_free (dirname);
 }
 
 static void
@@ -369,7 +463,7 @@ httpd_child_setup (gpointer user_data)
 static gboolean
 spawn_httpd (int port, pid_t *pid_out)
 {
-    char *free1, *free2;
+    char *free1, *free2, *free3;
     gboolean res;
     char *argv[10];
     char *env[10];
@@ -381,8 +475,11 @@ spawn_httpd (int port, pid_t *pid_out)
     gboolean got_pidfile;
     GConfClient *client;
     char *str;
+	char *public_dir;
 
-    ensure_public_dir ();
+	public_dir = lookup_public_dir ();
+	
+    ensure_public_dir (public_dir);
     ensure_conf_dir ();
 
     i = 0;
@@ -413,6 +510,7 @@ spawn_httpd (int port, pid_t *pid_out)
 
     i = 0;
     free2 = env[i++] = g_strdup_printf("HOME=%s", g_get_home_dir());
+    free3 = env[i++] = g_strdup_printf("XDG_PUBLICSHARE_DIR=%s", public_dir);
     env[i++] = "LANG=C";
     env[i] = NULL;
 
@@ -430,7 +528,9 @@ spawn_httpd (int port, pid_t *pid_out)
 			&error);
     g_free (free1);
     g_free (free2);
-    
+    g_free (free3);
+	g_free (public_dir);
+
     if (!res) {
 		fprintf (stderr, "error spawning httpd: %s\n",
 				 error->message);
